#include "../netproto.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include "net.h"


int loglevel=0;

struct Verb{
    const char* name;
    int numargs;
    int (*func)(char *);
};

struct Verb verbs[]={
    //daemon specific
    {"start", 1, sv_startservice},
    {"stop", 1, sv_stopservice},
    {"kill", 1, sv_killservice},
    {"enable", 1, sv_enableservice},
    {"disable", 1, sv_disableservice},
    {"restart", 1, sv_restart},
    {"status",1, sv_getstatus},
    //environment / daemon specific
    {"reload-daem", 0, sv_reloaddaem},
    {"sync-env", 0, sv_syncEnvironment},
};

int verbslen = sizeof(verbs)/sizeof(verbs[0]);

void help(){
    const char *helpstring =
    "Udaem client help - Protocol version: %i\n"
    "Usage: %s <verb> [arguments]\n"
    "\n"
    "Verbs:\n"
    "  Service Verbs (take a service as an argument):\n"
    "    start          start a service\n"
    "    stop           stop a service (SIGTERM)\n"
    "    kill           kill a service (SIGKILL)\n"
    "    enable         enable restarting and autostart of a service\n"
    "    disable        disable restarting and autostart of a service\n"
    "    restart        restart a service\n"
    "    status         get the current status of a service\n"
    "  Variables (no arguments):\n"
    "    sync-env       synchronise environment variables\n"
    "    reload-daem    reload all daemon configs\n"
    ;
    printf(helpstring, 1, "udactl");
    exit(EXIT_SUCCESS);
}

int main(int argc, char **argv){
    if(argc == 1){
        help();
    }else if(argc>1){
        struct Verb *verb=NULL;
        for (int i = 0; i<verbslen; i++) {
            if (strcmp(verbs[i].name, argv[1])==0) {
                verb=&verbs[i];
                printf("found command. cmd=%s\n",verbs[i].name);
            }
        }
        if(!verb){
            printf("Client: Verb \"%s\" not found. Run without args for help.\n", argv[1]);
            return 0;
        }

        if(!(argc-2 == verb->numargs || verb->numargs==-1)){
            printf("Not enough or too many args.\n");
            return 0;
        }

        unsigned int len = 0;
        for (int i = 2; i<argc; i++) {
            len += strlen(argv[i])+1;
        }

        char *singleStr = NULL;
        if(len>1) singleStr = malloc(len*sizeof(char));

        if(singleStr){
            char *t=singleStr;
            for (int i = 2; i<argc; i++) {
                strcpy(t, argv[i]);
                t += strlen(argv[i])+1;
            }
        }

        printf("\ncommand returned: %i\n",verb->func(singleStr));

        free(singleStr);


    }
}