#include "net.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>

struct Status sendToD(struct header head, char *body){
    int sockfd = socket(AF_UNIX, SOCK_STREAM, 0);

    struct sockaddr_un sa = {AF_UNIX,"udaem.sock"};
    long len = sizeof(sa);

    int res = connect(sockfd, (const struct sockaddr*)&sa, len);

    if(res!=0){
        printf("connect failed\n");
    }

    send(sockfd, (char*)&head, sizeof(head), 0);
    if(body)send(sockfd, body, head.datalength, 0);

    struct Status stat;

    recv(sockfd, (char*)&stat, sizeof(stat), 0);
    close(sockfd);

    return stat;
}

int sv_startservice(char *name){
    struct header head = {
        1,
        STARTSERVICE,
        (strlen(name)+1) * sizeof(char)
    };
    struct Status stat = sendToD(head, name);
    if (stat.err==OK) {
        return 1;
    }
    return -stat.err;
}

int sv_stopservice(char *name){
    struct header head = {
        1,
        STOPSERVICE,
        (strlen(name)+1) * sizeof(char)
    };
    struct Status stat = sendToD(head, name);
    if (stat.err==OK) {
        return 1;
    }
    return -stat.err;
}

int sv_killservice(char *name){
    struct header head = {
        1,
        KILLSERVICE,
        (strlen(name)+1) * sizeof(char)
    };
    struct Status stat = sendToD(head, name);
    if (stat.err==OK) {
        return 1;
    }
    return -stat.err;
}

int sv_enableservice(char *name){
    struct header head = {
        1,
        ENABLESERVICE,
        (strlen(name)+1) * sizeof(char)
    };
    struct Status stat = sendToD(head, name);
    if (stat.err==OK) {
        return 1;
    }
    return -stat.err;
}

int sv_disableservice(char *name){
    struct header head = {
        1,
        DISABLESERVICE,
        (strlen(name)+1) * sizeof(char)
    };
    struct Status stat = sendToD(head, name);
    if (stat.err==OK) {
        return 1;
    }
    return -stat.err;
}

char* getcurrentenv(unsigned long *len){
    char** i=__environ;
    unsigned long reqlen = 0;

    while(*i!=NULL){
        reqlen+=strlen(*i)+1;
        i++;
    }

    char* env = malloc(reqlen);

    i=__environ;
    char* pos=env;
    while(*i!=NULL){
        strcpy(pos, *i);
        pos+=strlen(*i)+1;
        i++;
    }
    *len=reqlen;
    return env;
    
}

int setEnvVars(char* envvars, unsigned long len){
    struct header head = {
        1,
        ENABLESERVICE,
        len
    };
    struct Status stat = sendToD(head, envvars);
    if (stat.err==OK) {
        return 1;
    }
    return -stat.err;
}

int sv_syncEnvironment(char *nreq){
    unsigned long len;
    char *env=getcurrentenv(&len);
    int ret = setEnvVars(env, len);
    free(env);
    return 1;
};

int sv_restart(char *name){
    struct header head = {
        1,
        RESTARTSERVICE,
        (strlen(name)+1) * sizeof(char)
    };
    struct Status stat = sendToD(head, name);
    if (stat.err==OK) {
        return 1;
    }
    return -stat.err;
}

int sv_reloaddaem(char *nreq){
    struct header head = {
        1,
        RELOADDAEMON,
        0
    };
    struct Status stat = sendToD(head, NULL);
    if (stat.err==OK) {
        return 1;
    }
    return -stat.err;

}

int sv_getstatus(char *name){
    struct header head = {
        1,
        STATUS,
        (strlen(name)+1) * sizeof(char)
    };
    // if (stat.err!=MOREDATA) {
    //     return -stat.err;
    // }

    int sockfd = socket(AF_UNIX, SOCK_STREAM, 0);

    struct sockaddr_un sa = {AF_UNIX,"udaem.sock"};
    long len = sizeof(sa);

    int res = connect(sockfd, (const struct sockaddr*)&sa, len);

    if(res!=0){
        printf("connect failed\n");
    }

    send(sockfd, (char*)&head, sizeof(head), 0);
    if(name)send(sockfd, name, head.datalength, 0);

    struct Status stat;

    recv(sockfd, (char*)&stat, sizeof(stat), 0);

    if(stat.err != MOREDATA){
        return -stat.err;
    }

    struct header rhead = {0};

    recv(sockfd, &rhead, sizeof(struct header), 0);

    if(rhead.type != STATUS){
        close(sockfd);
        return -1;        
    }

    char buf[rhead.datalength];
    
    recv(sockfd, buf, rhead.datalength, 0);
    
    char *ps[5];
    char *p=buf;
    int complete=1;

    for (int i=0; i<5 && p<buf+rhead.datalength-sizeof(int)*3; i++) {
        if(*p){ps[i]=p; p+=strlen(ps[i])+1;}
        else {ps[i]=0; complete=0; p+=2;}
    }

    int state;
    memcpy(&state, p, sizeof(int));
    p+=sizeof(int);

    int pid;
    memcpy(&pid, p, sizeof(int));
    p+=sizeof(int);
    
    short exitreason=0;
    memcpy(&exitreason, p, sizeof(short));
    p+=sizeof(short);

    short exitstatus=0;
    memcpy(&exitstatus, p, sizeof(short));

    if(ps[0]) printf("showing status for service: %s\n",ps[0]);
    else printf("WARNING: service is not named, this is probably a bug in the server.");

    printf("state: %i\n",state);

    if (state & RUNNING) {
        printf("Daemon running with pid: %i\n", pid);
    }else if(state & EXITED){
        printf("Daemon exited with status: %i\n", exitstatus);
    }else {
        printf("Daemon wasnt started this boot.\n");
    }

    if(ps[1]) printf("exec: %s\n", ps[1]);
    if(ps[2]) printf("argv: %s\n", ps[2]);
    if(ps[3]) printf("log: %s\n", ps[3]);
    if(ps[4]) printf("errlog: %s\n", ps[4]);



    close(sockfd);

    return 1;
}

