#include "daem.h"
#include "configs.h"
#include <asm-generic/errno-base.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>
#include <unistd.h>


// TODO: make this function less buggy and horribly written
void readDirs(struct Daemon **edlist, unsigned int *edlen, char* daemdir){
    DIR *dir;
    struct dirent *ent;
    dir=opendir(daemdir);

    unsigned int dlen=0;

    if(dir){
        while ((ent = readdir(dir)) != NULL ) {
            if(ent->d_name[0]=='.') continue;
            unsigned long slen=strlen(ent->d_name);
            if(slen<=10) continue;
            if(strcmp(&ent->d_name[slen-10], ".daem.conf") != 0){
                if(loglevel>0)printf("skipped %s.\n",ent->d_name);
                continue;
            }
            dlen++;
        }

        if(dlen==0){return;}

        rewinddir(dir);
        struct Daemon *daemons = (struct Daemon*)malloc(dlen*sizeof(struct Daemon));
        unsigned int i=0;
        while ((ent = readdir(dir)) != NULL && i<dlen) {
            if(ent->d_name[0]=='.') continue;
            unsigned long slen=strlen(ent->d_name);
            if(slen<=10) continue;
            if(strcmp(&ent->d_name[slen-10], ".daem.conf") != 0) continue;
            char path[strlen(daemdir) + slen + 1];

            strcpy(path, daemdir);
            strcpy(path+strlen(daemdir), ent->d_name);

            int err=0;
            daemons[i]=readDaemonCfg(path, &err);
            if(err && loglevel>0) printf("error loading daemon file with path: %s\n", path);

            i++;
        }
        *edlist=daemons;
        *edlen=dlen;
    }else {
        printf("could not open config or daemon dir\n");
        exit(EXIT_FAILURE);
    }
}

void handleReopenErr(const char *fdname, const char *file, int errnoval){
    switch(errnoval){
        case EACCES:{
            fprintf(stderr, "FATAL: could not reroute %s to %s , Access denied\n", fdname, file);
            break;
        }
        case EBADF:{
            fprintf(stderr, "FATAL: could not reroute \"%s\" to \"%s\" , Bad Filename\n", fdname, file);
            break;
        }
        case ENOENT:{
            fprintf(stderr, "FATAL: could not reroute \"%s\" to \"%s\" , No such directory\n", fdname, file);
            break;
        }
        default:{
            fprintf(stderr, "FATAL: could not reroute %s to %s , Something went very wrong. errno = %i;\n", fdname, file, errnoval);
            break;
        }
    }
    exit(EXIT_FAILURE);
}

int startProgram(const char* path, const char* arg, const char* stdoutfile, const char* stderrfile, int sleepd){
    int res = fork();
    if(res==0){
        sleep(sleepd);

        if(loglevel>1)printf("starting prog with path: %s \n",path);
        if(loglevel>2)printf("starting prog with args: %s \n",arg);


        // redirect output to file and (TODO: ) input to fifo
        FILE *fp;
        errno = 0;
        fp = freopen(stdoutfile, "a+", stdout);
        if(fp == NULL) handleReopenErr("stdout", stdoutfile, errno);
        fp = freopen(stderrfile, "a+", stderr); 
        if(fp == NULL) handleReopenErr("stderr", stderrfile, errno);

        // log that program is starting
        if(loglevel>1) printf("====        STARTING PROGRAM %s      ====\n", path);
        fflush(stdout);

        // concatenate program file path and arguments for shell
        int pathlen = strlen(path);
        char *combined = malloc(pathlen + strlen(arg) + 2);
        strcpy(combined, path);
        combined[pathlen] = ' ';
        strcpy(combined + pathlen + 1, arg);

        // execute the program
        int ret = execl(cfgptr->shell, cfgptr->shell, "-c", combined, NULL);

        // only gets executed when program fails
        if(loglevel>1) printf("==== PROGRAM (%s) EXITED WITH CODE %i ====\n", path, ret);
        fclose(stdout);
        fclose(stderr);
        exit(ret);
    }else if (res < 0){
        if(loglevel>0) printf("fork failed.\n");
        return -1;
    }else {
        return res;
    }
}

int startDaemon(struct Daemon *daemon, int sleepd){
    if(!(daemon->configflags & ISCOMPLETE)) return -1;
    if(daemon->state & RUNNING) return -2;
    daemon->state|=RUNNING;
    daemon->state&=~EXITED;
    int pid = startProgram(daemon->exec, daemon->argv, daemon->logf, daemon->errf, sleepd);
    daemon->pid=pid;
    return pid;
}

void killDaemon(struct Daemon *daemon, int sig){
    if(!daemon->pid) return;
    kill(daemon->pid, sig);
    daemon->pid=0;
}

void killDaemonWait(struct Daemon *daemon, int sig, unsigned int timeoutms){
    if(!daemon->pid) return;
    kill(daemon->pid, sig);
    int time=0;
    int status;
    int normexit=0;
    while (time<timeoutms) {
        status=0;
        if(waitpid(daemon->pid, &status, WNOHANG)){
            daemon->state&=~RUNNING;
            daemon->state|=EXITED;
            daemon->exitstatus=WEXITSTATUS(status);
            daemon->exitreason=WTERMSIG(status);
            normexit=1;
            break;
        }
        usleep(1000);
        time++;
    }
    if(!normexit){
        kill(daemon->pid, SIGKILL);
    }
    if(loglevel>1 && normexit) printf("killed process %s (%s), exit code: %i.\n", daemon->name, daemon->exec, daemon->exitstatus);
    if(loglevel>1 && !normexit) printf("forcefully killed process %s (%s), exit code: %i.\n", daemon->name, daemon->exec, daemon->exitstatus);
}

void restartDaemon(struct Daemon *daemon, int sig, unsigned int timeoutms){
    // clear restartrequired flag
    daemon->state &= ~RESTARTREQUIRED;
    // daemon is incomplete
    if(!(daemon->configflags & ISCOMPLETE)) return;
    killDaemonWait(daemon, sig, timeoutms);
    startDaemon(daemon, 0);
}

struct Daemon *findDaemon(char *name, struct Daemon *daemonlist, unsigned int daemonNum){
    for (int i=0; i<daemonNum; i++) {
        if(!(daemonlist[i].configflags & ISCOMPLETE)) continue;
        if(strcmp(daemonlist[i].name, name)==0){
            return &daemonlist[i];
        }
    }
    return NULL;
}

void reloadDaemons(struct Daemon **daemonlist, unsigned int daemonNum, char *daemdir){
    if(loglevel > 0) printf("reloading daemons...\n");
    struct Daemon *dlist;
    unsigned int dlen=0;
    readDirs(&dlist, &dlen, daemdir);

    for (unsigned int i=0; i<dlen; i++) {
        struct Daemon *daem = findDaemon(dlist[i].name, *daemonlist, daemonNum);
        if(daem){
            dlist[i] = mergeDaemonToFree(dlist[i], *daem);
        }
    }
    cleanDaemonlist(*daemonlist, daemonNum);
    free(*daemonlist);
    *daemonlist=dlist;
    if(loglevel > 0) printf("daemons reloaded.\n");
}

void checkOnDaemons(struct Daemon *daemonlist, unsigned int daemonNum){
    for (int i=0; i<daemonNum; i++) {

        if (daemonlist[i].state & RESTARTREQUIRED) {
            restartDaemon(&daemonlist[i], SIGTERM, 500);
        }

        int status;
        if(!daemonlist[i].pid) continue;
        if(waitpid(daemonlist[i].pid, &status, WNOHANG)){
            if(!(daemonlist[i].state & RUNNING)) continue;
            daemonlist[i].state&=~RUNNING;
            daemonlist[i].state|=EXITED;
            daemonlist[i].exitstatus=WEXITSTATUS(status);
            daemonlist[i].exitreason=WTERMSIG(status);
            daemonlist[i].pid=0;
            if(loglevel>1) printf("process: %s (%s) exited with code: %i.\n", daemonlist[i].name, daemonlist[i].exec, daemonlist[i].exitstatus);
            
            if(daemonlist[i].exitstatus != 0 && daemonlist[i].configflags & RESTART_ON_CRASH && daemonlist[i].state & ENABLE){
                startDaemon(&daemonlist[i], daemonlist[i].restartinterval);
            }

            if(daemonlist[i].exitstatus == 0 && daemonlist[i].configflags & RESTART_ON_EXIT && daemonlist[i].state & ENABLE){
                startDaemon(&daemonlist[i], daemonlist[i].restartinterval);
            }
        }
    }
}

void cleanDaemonlist(struct Daemon *daemonlist, unsigned int daemonNum){
    for (int i=0; i<daemonNum; i++) {
        struct Daemon *d = &daemonlist[i];
        if(d->argv) {free(d->argv); d->argv = NULL;}
        if(d->errf) {free(d->errf); d->errf = NULL;}
        if(d->exec) {free(d->exec); d->exec = NULL;}
        if(d->logf) {free(d->logf); d->logf = NULL;}
        if(d->name) {free(d->name); d->name = NULL;}
    }
}