#ifndef __DAEM_H__
#define __DAEM_H__
#include <string.h>
#include <dirent.h>
#include <stdlib.h>
#include <stdio.h>

#define SHELL "/bin/sh";

extern int loglevel;

struct Daemon{
    char* name;
    char* exec;
    char* argv;
    char* logf;
    char* errf;
    
    int configflags;
    int restartinterval;
    
    int state;
    int pid;
    short int exitstatus;
    short int exitreason;
};

/*
 * returns 1 when the daemon has the minimum required to start
 * otherwise returns 0
 */
int daemonIsComplete(struct Daemon *daem);

/*
 * read a directory with daemons inside
 */
void readDirs(struct Daemon **edlist, unsigned int *edlen, char *daemdir);

/*
 * reloads all daemons in ram from disk, setting the RESTARTREQUIRED or RESTARTRECOMENDED flags if necessary
 */
void reloadDaemons(struct Daemon **daemonlist, unsigned int daemonNum, char *daemdir);


/*
 * start a program with specific stdout, stderr and a parameter for a sleep delay in seconds
 */
int startProgram(const char* path, const char* arg, const char* stdoutfile, const char* stderrfile, int sleepd);

/*
 * start a daemon with a sleep delay
 */
int startDaemon(struct Daemon *daemon, int sleepd);


/*
 * kill a daemon without waiting for it to exit
 */
void killDaemon(struct Daemon *daemon, int sig);

/*
 * kill a daemon and wait with a timeout in milliseconds
 */
void killDaemonWait(struct Daemon *daemon, int sig, unsigned int timeoutms);

/*
 * restart a daemon with a timeout in milliseconds and a signal to kill it with
 */
void restartDaemon(struct Daemon *daemon, int sig, unsigned int timeoutms);



// -------------------------------------------------------------------------
// | DAEMON MANAGEMENT
// ---------------

/*
 * check on the daemons
 * restarts daemon if required, user has to restart daemons if recomended
 */
void checkOnDaemons(struct Daemon *daemonlist, unsigned int daemonNum);

/*
 * find a daemon by name
 */
struct Daemon *findDaemon(char *name, struct Daemon *daemonlist, unsigned int daemonNum);

/*
 * clean a daemonlist
 */
void cleanDaemonlist(struct Daemon *daemonlist, unsigned int daemonNum);

#endif