#include "config_struct.h"
#include "daem.h"
#include "net.h"
#include "configs.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h> 
#include <wordexp.h>
#include "helpers.h"
#include "../configuration.h"

struct Daemon *daemonlist;
struct Config *cfgptr;
unsigned int daemonNum;


int sockfd=0;
// 0 = log only fatal errors
// 1 = log minor errors
// 2 = log starting and stopping of daemons
// 3 = log rare events
// 4 = log debugging info
int loglevel = 4;

void onterm(int sig){
    if (sockfd) {
        close(sockfd);
        unlink("udaem.sock");
    }
    cleanDaemonlist(daemonlist, daemonNum);
    printf("\ngoodbye!\n");
    exit(EXIT_SUCCESS);
}

void sigpipe(int sig){
    printf("sigpipe ):\n");
}

void incompleteConfig(char *filename){
    fprintf(stderr, "config file \"%s\" incomplete.\n", filename);
    exit(EXIT_FAILURE);
}

int main(){
    signal(SIGTERM, onterm);
    signal(SIGINT, onterm);
    signal(SIGPIPE, sigpipe);

    int err = 0;
    struct Config cfg;

    // test for $XDG_CONFIG_HOME/udaem/udaem.conf
    err = 0;
    char *cfghome;
    if((cfghome = getenv("XDG_CONFIG_HOME"))){
        wordexp_t p;
        // TODO: make this better
        wordexp(cfghome, &p, WRDE_NOCMD);

        char *path = concatenateString(p.we_wordv[0], "/udaem/udaem.conf");
        printf("trying %s (XDG_CONFIG_HOME)...\n", path);

        cfg = readSelfCfg(path, &err);
        if(err==2){incompleteConfig(path);}
        if(err==0){goto afterconfig;}
    }

    // test for $HOME/.config/udaem/udaem.conf
    err = 0;
    char *path = concatenateString(getenv("HOME"), "/.config/udaem/udaem.conf");
    printf("trying %s (.config)...\n", path);
    cfg = readSelfCfg(path, &err);
    if(err==2){incompleteConfig(path);}
    free(path);
    if(err==0){goto afterconfig;}

    // test for /etc/udaem/udaem.conf
    cfg = readSelfCfg("/etc/udaem/udaem.conf", &err);
    printf("trying %s ...\n", "/etc/udaem/udaem.conf");
    if(err==2){incompleteConfig("/etc/udaem/udaem.conf");}
    if(err==0){goto afterconfig;}

    #ifdef DEBUG
    // test for the debug daemon configuration
    err = 0;
    cfg = readSelfCfg("config/udaem.conf", &err);
    if(err==2){incompleteConfig("config/udaem.conf");}
    if(err==0){goto afterconfig;}
    #endif

    fprintf(stderr, "FATAL: Daemon config not found!\n");
    exit(EXIT_FAILURE);
afterconfig:
    cfgptr = &cfg;
    
    loglevel = cfg.loglevel;
    printf("loglevel: %i\n", loglevel);

    readDirs(&daemonlist, &daemonNum, cfg.daemdir);
    commloop(&daemonlist, &daemonNum, &sockfd, &cfg);
}
