#define _GNU_SOURCE
#include "config_struct.h"
#include "configs.h"
#include "net.h"
#include "netproto.h"
#include "daem.h"
#include <signal.h>
#include <sys/socket.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

void sendStatus(int cfd, enum errortype et){
    struct Status err={API_VERSION, et};
    send(cfd, (char*)&err, sizeof(struct Status), 0);
}

void commhandler(int cfd, char *buffer, enum msgtype type, unsigned long bufferlen, struct Daemon **daemonlist, unsigned int *daemonNum, struct Config *conf){
    switch (type) {
        // start a service (ignore enable status)
        case STARTSERVICE:{
            struct Daemon* daem = findDaemon(buffer, *daemonlist, *daemonNum);
            if (daem) {
                int ret = startDaemon(daem, 0);
                if(ret == -1) sendStatus(cfd, SERVICEINCOMPLETE);
                if(ret == -2) sendStatus(cfd, SERVICERUNNING);
            }else {
                sendStatus(cfd, NOSUCHSERVICE);
                break;
            }
            sendStatus(cfd, OK);
            break;
        }
        // stop and disable a service (to prevent restarts)
        case STOPSERVICE:{
            struct Daemon* daem = findDaemon(buffer, *daemonlist, *daemonNum);
            if (daem) {
                killDaemonWait(daem, SIGTERM, 1000);
            }else {
                sendStatus(cfd, NOSUCHSERVICE);
                break;
            }
            sendStatus(cfd, OK);
            break;
        }
        // kill the daemon with signal 9
        case KILLSERVICE:{
            struct Daemon* daem = findDaemon(buffer, *daemonlist, *daemonNum);
            if (daem){
                killDaemon(daem, SIGKILL);
            }else {
                sendStatus(cfd, NOSUCHSERVICE);
                break;
            }
            sendStatus(cfd, OK);
            break;
        }
        case RESTARTSERVICE:{
            struct Daemon* daem = findDaemon(buffer, *daemonlist, *daemonNum);
            if (daem){
                restartDaemon(daem, SIGTERM, 2000);
            }else {
                sendStatus(cfd, NOSUCHSERVICE);
                break;
            }
            sendStatus(cfd, OK);
            break;
        }
        // enable automatic restarts (if configured) for a service
        case ENABLESERVICE:{
            struct Daemon* daem = findDaemon(buffer, *daemonlist, *daemonNum);
            if (daem) {
                daem->state|=ENABLE;
            }else {
                sendStatus(cfd, NOSUCHSERVICE);
                break;
            }
            sendStatus(cfd, OK);
            break;
        }
        // disable automatic restarts for a service
        case DISABLESERVICE:{
            struct Daemon* daem = findDaemon(buffer, *daemonlist, *daemonNum);
            if (daem) {
                daem->state&=~ENABLE;
            }else {
                sendStatus(cfd, NOSUCHSERVICE);
                break;
            }
            sendStatus(cfd, OK);
            break;
        }

        case STATUS:{
            struct Daemon* daem = findDaemon(buffer, *daemonlist, *daemonNum);
            if (daem) {
                sendStatus(cfd, MOREDATA);
                
                long len = 0;

                if(daem->name) len+=(strlen(daem->name)+1); else len+=2;
                if(daem->exec) len+=(strlen(daem->exec)+1); else len+=2;
                if(daem->argv) len+=(strlen(daem->exec)+1); else len+=2;
                if(daem->logf) len+=(strlen(daem->logf)+1); else len+=2;
                if(daem->errf) len+=(strlen(daem->errf)+1); else len+=2;

                len+=sizeof(daem->state);
                len+=sizeof(daem->pid);
                len+=sizeof(daem->exitstatus);
                len+=sizeof(daem->exitreason);

                char buf[len];
                memset(buf, 0, len * sizeof(char));
                char *p;

                p=buf;

                if(daem->name){strcpy(p, daem->name); p+=strlen(daem->name)+1;} else {*p=0; p++; p=0; p++;}
                if(daem->exec){strcpy(p, daem->exec); p+=strlen(daem->exec)+1;} else {*p=0; p++; p=0; p++;}
                if(daem->argv){strcpy(p, daem->argv); p+=strlen(daem->argv)+1;} else {*p=0; p++; p=0; p++;}
                if(daem->logf){strcpy(p, daem->logf); p+=strlen(daem->logf)+1;} else {*p=0; p++; p=0; p++;}
                if(daem->errf){strcpy(p, daem->errf); p+=strlen(daem->errf)+1;} else {*p=0; p++; p=0; p++;}

                memcpy(p, &daem->state, sizeof(daem->state));
                p+=sizeof(daem->state);

                memcpy(p, &daem->pid, sizeof(daem->pid));
                p+=sizeof(daem->pid);

                memcpy(p, &daem->exitstatus, sizeof(daem->exitreason));
                p+=sizeof(daem->exitstatus);
                
                memcpy(p, &daem->exitreason, sizeof(daem->exitreason));
                p+=sizeof(daem->exitreason);

                struct header head = {1, STATUS, len};
                send(cfd, (char*)&head, sizeof(head), 0);
                send(cfd, buf, sizeof(buf), 0);

            }else {
                sendStatus(cfd, NOSUCHSERVICE);
                break;
            }
            sendStatus(cfd, OK);
            break;
        }

        // set a list of environment variables
        case SETENV:{
            unsigned long pos=0;
            while (pos<bufferlen) {
                char *pk = &buffer[pos];
                putenv(pk);
                pos += strlen(pk) + 1;                
            }
            sendStatus(cfd, OK);
            break;
        }

        // reload master daemon
        case RELOADDAEMON:{
            reloadDaemons(daemonlist, *daemonNum, conf->daemdir);
            sendStatus(cfd, OK);
            break;
        }

        default:{
            sendStatus(cfd, NOTIMPLEMENTED);
            break;
        }
    }
}

void handleconn(int cfd, struct Daemon **daemonlist, unsigned int *daemonNum, struct Config *conf){

    struct header message;
    long res = recv(cfd, &message, sizeof(struct header), 0);
    if(loglevel>3)printf("recvmsg:\n v=%u\n typ=%i\n len=%lu\n", message.version, message.type, message.datalength);
    int currupt=1;
    if(message.version==1 && message.type<MAXMSGTYPE){
        if(message.datalength>0){
            char buffer[message.datalength];
            long rl = recv(cfd, buffer, message.datalength, 0);
            if(buffer[message.datalength-1]==0 && rl==message.datalength){
                currupt = 0;
                commhandler(cfd, buffer, message.type, message.datalength, daemonlist, daemonNum, conf);
            }
        }else if(message.datalength == 0){
            currupt = 0;
            commhandler(cfd, "", message.type, 0, daemonlist, daemonNum, conf);
        }
    }

    if(currupt) {send(cfd, "CRM currupt message, or wrong version", 38, 0);}

    close(cfd);
}

void commloop(struct Daemon **daemonlist, unsigned int *daemonNum, int *sockfd, struct Config *conf){
    *sockfd = socket(AF_UNIX, SOCK_STREAM | SOCK_NONBLOCK, 0);

    struct sockaddr_un sa = {
        AF_UNIX,
        "udaem.sock"
    };
    unsigned int size=sizeof(sa);

    int res = bind(*sockfd, (const struct sockaddr*)&sa, size);

    if(res!=0){
        printf("bind failed\n");
        exit(EXIT_FAILURE);
    }
    
    res = listen(*sockfd, 1);
    if(res!=0){
        printf("listen failed\n");
        exit(EXIT_FAILURE);
    }

    while (1) {
        int cfd = accept4(*sockfd, (struct sockaddr*)&sa, &size, SOCK_NONBLOCK);
        if(cfd>=0){
            handleconn(cfd, daemonlist, daemonNum, conf);
        }
        checkOnDaemons(*daemonlist, *daemonNum);
        usleep(conf->loopinterval*1000);
    }
}
