#include <math.h>
#include <security/_pam_types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define PAM_SM_SESSION
#include <security/pam_appl.h>
#include <security/pam_misc.h>
#include <security/pam_modules.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>

int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, const char **argv){
    pam_putenv(pamh, "RAILGUN=MISAKA");  

    
    // Get the username of the logging in user
    const char* usr;
    int res = pam_get_user(pamh, &usr, NULL);
    if(res != PAM_SUCCESS || !usr){
        printf("Failed to get pam user, thoust environment may be misconfigured.\n");
        return PAM_IGNORE;
    }

    // Get the user id of the logging in user
    struct passwd *pw;
    pw = getpwnam(usr);
    if (pw == NULL) {
        printf("Failed to get user id, thoust environment may be misconfigured.\n");
        return PAM_IGNORE;
    }

    unsigned int uid = pw->pw_uid;
    unsigned int gid = pw->pw_gid;

    printf("User: %s  UID:%i\n", usr, uid);

    char * pathname = "/run/user/";

    char * runtimePath;

    int s = asprintf(&runtimePath, "%s%u", pathname, uid);
    if (s<0) {
        printf("asprintf() failed, toust environment may be misconfigured.\n");
        return PAM_IGNORE;
    }

    printf("RuntimePath: %s\n", runtimePath);
    
    struct stat st = {0};
    if(stat(runtimePath, &st) == -1){
        printf("%s doesn't exist, creating...\n", runtimePath);
        mkdir(runtimePath, 0700);
        chown(runtimePath, uid, gid);
    }else {
        printf("%s exists.\n", runtimePath);
    }
 
    const char *runtimeEnvVar = pam_getenv(pamh, "XDG_RUNTIME_DIR");

    if(runtimeEnvVar && strcmp(runtimeEnvVar, runtimePath) == 0){
        printf("Runtime envvar allready set up.");
    }else {
        printf("Setting up runtime environment var.\n");
        char *runtimeDirEnvVar;
        asprintf(&runtimeDirEnvVar, "XDG_RUNTIME_DIR=%s", runtimePath);

        pam_putenv(pamh, runtimeDirEnvVar);
        free(runtimeDirEnvVar);
    }

    free(runtimePath);
    
    int pid = fork();
    if(pid < 0){
        printf("Failed to start Daemon, fork().\n");
        return PAM_IGNORE;
    }else if(pid == 0){
        // start daemon
        // see here https://github.com/linux-pam/linux-pam/blob/master/modules/pam_exec/pam_exec.c#L325
        exit(EXIT_FAILURE);
    }

    return PAM_IGNORE;
}

int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc, const char **argv){
    return PAM_IGNORE;
}